package org.hso.ecommerce.controller.intern.customers;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import org.hso.ecommerce.api.RestServiceForDelivery;
import org.hso.ecommerce.entities.shop.CustomerOrder;

import org.hso.ecommerce.repos.shop.CustomerOrderRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.datetime.DateFormatterRegistrar;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.transaction.Transaction;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Controller
@RequestMapping("intern/customerOrders")
public class CustomerOrderController {

    @Autowired
    private final CustomerOrderRepository customerOrderRepository = null;
    @Autowired
    private final RestServiceForDelivery restServiceForDelivery = null;

    @GetMapping("")
    public String internCustomerOrder(Model model) {
        List<CustomerOrder> orders =  customerOrderRepository.getAllOrders();

        Map<CustomerOrder, DeliveryData> customerOrderDeliveryDataMap = orders.stream().collect
                (Collectors.toMap
                (Function.identity(), this::getDeliveryDataFromCustomerOrder));

        model.addAttribute("orderDeliveryDataMap", customerOrderDeliveryDataMap);
        return "intern/customerOrders/index";
    }

    @GetMapping("/{id}")
    public String internCustomerOrdersId(Model model,
                                         @PathVariable("id") String id
    ) {
        CustomerOrder order = customerOrderRepository.findById(Long.parseLong(id)).get();
        DeliveryData deliveryData = getDeliveryDataFromCustomerOrder(order);

        model.addAttribute("order", order);
        model.addAttribute("deliveryData", deliveryData);

        return "intern/customerOrders/id";
    }

    private  DeliveryData getDeliveryDataFromCustomerOrder(CustomerOrder customerOrder)
    {
        if(customerOrder.trackingId == null)
            return new CustomerOrderController.DeliveryData("Bestellung wurde elektronisch angekündigt","");

        if(customerOrder.deliveredAt == null)
        {
            DeliveryData deliveryData = restServiceForDelivery.getDeliveryData(UUID.fromString(customerOrder.trackingId));

            if(deliveryData.isDelivered())
            {
                Calendar calendar = Calendar.getInstance();

                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd-MM-yyyy");

                try {
                    calendar.setTime(simpleDateFormat.parse(deliveryData.estimatedArrival));
                } catch (ParseException e) {
                    e.printStackTrace();
                }

                customerOrderRepository.updateUserDeliveredAt(customerOrder.id, new Timestamp(calendar.getTimeInMillis()));
            }

            return deliveryData;
        }

        SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-yyyy");
        return new CustomerOrderController.DeliveryData("Lieferung erfolgreich",formatter.format(customerOrder.deliveredAt));
    }

    public static class DeliveryData
    {

        private final String status;

        private final String estimatedArrival;

        private boolean isDelivered;

        @JsonCreator(mode = JsonCreator.Mode.PROPERTIES)
        public DeliveryData(String status, String estimatedArrival) {
            this.status = status;
            this.estimatedArrival = estimatedArrival;
            isDelivered = status.equals("Lieferung erfolgreich");
        }

        public boolean isDelivered() {
            return isDelivered;
        }

        public String getStatus() {
            return status;
        }

        public String getEstimatedArrival() {
            return estimatedArrival;
        }
    }

}
